using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using ADSLibrary;
using ADSUtil;
using System.Threading;

namespace ADSIO
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Panel sioPanel;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label firmwareLabel;
		private System.Windows.Forms.Label deviceversionLabel;
		private System.Windows.Forms.Label devicepartLabel;
		
		private GMasterJ7 J7;
		private GMasterJ2 J2;

		private System.Windows.Forms.Timer appTimer;

		public Form1()
		{
			InitializeComponent();

			J7 = new GMasterJ7();
			J7.Location = new Point(0,0);
			J7.PinClick += new PinClickHandler(OnPinClick);

			J2 = new GMasterJ2();
			J2.Location = new Point(200,0);
			J2.PinClick += new PinClickHandler(OnPinClick);

			this.Controls.Add(J7);
			this.Controls.Add(J2);
			
			// configure ADSmartIO for Analog inputs on Port A
			ADSmartIO.SelectOption(3);
			ADSmartIO.InitDDR(ADSmartIO.PortName.A, 0);
			
			appTimer = new System.Windows.Forms.Timer();

			this.RefreshPins();
			appTimer.Tick += new EventHandler(TimerTick);
			appTimer.Interval = 250;
			appTimer.Enabled = true;
		}

		private void TimerTick(object sender, System.EventArgs e)
		{
			this.RefreshPins();
		}

		public void RefreshPins()
		{
			// refresh J2
			try
			{
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO0 - 1].Value = CEPorts.ReadPortByte("IOA1:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO1 - 1].Value = CEPorts.ReadPortByte("IOA2:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO2 - 1].Value = CEPorts.ReadPortByte("IOA3:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO3 - 1].Value = CEPorts.ReadPortByte("IOA4:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO4 - 1].Value = CEPorts.ReadPortByte("IOA5:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO5 - 1].Value = CEPorts.ReadPortByte("IOB1:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO6 - 1].Value = CEPorts.ReadPortByte("IOB2:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO7 - 1].Value = CEPorts.ReadPortByte("IOB3:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO8 - 1].Value = CEPorts.ReadPortByte("IOB4:");
				J2.Pins[(byte)GMasterJ2.PinName.UCB_IO9 - 1].Value = CEPorts.ReadPortByte("IOB5:");

				J2.Pins[(byte)GMasterJ2.PinName.ANIN0 - 1].Value = CEPorts.ReadPortWord("AIN1:");
				J2.Pins[(byte)GMasterJ2.PinName.ANIN1 - 1].Value = CEPorts.ReadPortWord("AIN2:");
				J2.Pins[(byte)GMasterJ2.PinName.ANIN2 - 1].Value = CEPorts.ReadPortWord("AIN3:");
				J2.Pins[(byte)GMasterJ2.PinName.ANIN3 - 1].Value = CEPorts.ReadPortWord("AIN4:");

				J2.Pins[(byte)GMasterJ2.PinName.PB0 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.B) & 1) == 1 ? (uint)1 : 0;
				J2.Pins[(byte)GMasterJ2.PinName.PB1 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.B) & 2) == 2 ? (uint)1 : 0;
				J2.Pins[(byte)GMasterJ2.PinName.PB2 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.B) & 4) == 4 ? (uint)1 : 0;
				J2.Pins[(byte)GMasterJ2.PinName.PB3 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.B) & 8) == 8 ? (uint)1 : 0;

				J2.Pins[(byte)GMasterJ2.PinName.PD0 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.D) & 1) == 1 ? (uint)1 : 0;
				J2.Pins[(byte)GMasterJ2.PinName.PD1 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.D) & 2) == 2 ? (uint)1 : 0;

				// refresh J7
				J7.Pins[(byte)GMasterJ7.PinName.PA0 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 1) == 1 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA1 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 2) == 2 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA2 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 4) == 4 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA3 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 8) == 8 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA4 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 16) == 16 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA5 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 32) == 32 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA6 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 64) == 64 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PA7 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.A) & 128) == 128 ? (uint)1 : 0;

				J7.Pins[(byte)GMasterJ7.PinName.PC0 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 1) == 1 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC1 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 2) == 2 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC2 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 4) == 4 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC3 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 8) == 8 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC4 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 16) == 16 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC5 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 32) == 32 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC6 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 64) == 64 ? (uint)1 : 0;
				J7.Pins[(byte)GMasterJ7.PinName.PC7 - 1].Value = (ADSmartIO.ReadPort(ADSmartIO.PortName.C) & 128) == 128 ? (uint)1 : 0;
			}
			catch(Exception ex)
			{
				if(appTimer != null) 
					appTimer.Enabled = false;

				Console.WriteLine("Exception updating pins: " + ex.Message);
			}
		}

		private void OnPinClick(Pin sender, PinClickEventArgs e)
		{
			// if it's digital, toggle it's value
			// otherwise change swap the text/value display
			if(sender.Type == PinType.Digital)
			{
				if(sender.Parent.Equals(J2))
				{
					try
					{
						#region switch((GMasterJ2.PinName)sender.PinNumber)
						switch((GMasterJ2.PinName)sender.PinNumber)
						{
							case GMasterJ2.PinName.UCB_IO0:
								CEPorts.WritePort("IOA1:", sender.Value == 0 ? (byte)1 : (byte)0);
								break;
							case GMasterJ2.PinName.UCB_IO9:
								CEPorts.WritePort("IOB5:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO1:
								CEPorts.WritePort("IOA2:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO8:
								CEPorts.WritePort("IOB4:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO2:
								CEPorts.WritePort("IOA3:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO7:
								CEPorts.WritePort("IOB3:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO3:
								CEPorts.WritePort("IOA4:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO6:
								CEPorts.WritePort("IOB2:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO4:
								CEPorts.WritePort("IOA5:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.UCB_IO5:
								CEPorts.WritePort("IOB1:", Convert.ToByte(sender.Value));
								break;
							case GMasterJ2.PinName.PB0:
								ADSmartIO.WritePort(ADSmartIO.PortName.B, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.B)) ^ 1));
								break;
							case GMasterJ2.PinName.PB1:
								ADSmartIO.WritePort(ADSmartIO.PortName.B, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.B)) ^ (1 << 1)));
								break;
							case GMasterJ2.PinName.PB2:
								ADSmartIO.WritePort(ADSmartIO.PortName.B, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.B)) ^ (1 << 2)));
								break;
							case GMasterJ2.PinName.PB3:
								ADSmartIO.WritePort(ADSmartIO.PortName.B, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.B)) ^ (1 << 3)));
								break;
							case GMasterJ2.PinName.PD0:
								ADSmartIO.WritePort(ADSmartIO.PortName.D, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.D)) ^ 1));
								break;
							case GMasterJ2.PinName.PD1:
								ADSmartIO.WritePort(ADSmartIO.PortName.D, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.D)) ^ (1 << 1)));
								break;
							default:
								break;
						}
						#endregion
					}
					catch(Exception ex)
					{
						Console.WriteLine("Exception: " + ex.Message);
					}
				}
				else
				{
					try
					{
						#region switch((GMasterJ7.PinName)sender.PinNumber)
						switch((GMasterJ7.PinName)sender.PinNumber)
						{
							case GMasterJ7.PinName.PA0:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ 1));
								break;
							case GMasterJ7.PinName.PA1:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 1)));
								break;
							case GMasterJ7.PinName.PA2:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 2)));
								break;
							case GMasterJ7.PinName.PA3:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 3)));
								break;
							case GMasterJ7.PinName.PA4:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 4)));
								break;
							case GMasterJ7.PinName.PA5:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 5)));
								break;
							case GMasterJ7.PinName.PA6:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 6)));
								break;
							case GMasterJ7.PinName.PA7:
								ADSmartIO.WritePort(ADSmartIO.PortName.A, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.A)) ^ (1 << 7)));
								break;
							case GMasterJ7.PinName.PC0:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ 1));
								break;
							case GMasterJ7.PinName.PC1:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 1)));
								break;
							case GMasterJ7.PinName.PC2:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 2)));
								break;
							case GMasterJ7.PinName.PC3:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 3)));
								break;
							case GMasterJ7.PinName.PC4:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 4)));
								break;
							case GMasterJ7.PinName.PC5:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 5)));
								break;
							case GMasterJ7.PinName.PC6:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 6)));
								break;
							case GMasterJ7.PinName.PC7:
								ADSmartIO.WritePort(ADSmartIO.PortName.C, (byte)(Convert.ToByte(ADSmartIO.ReadPort(ADSmartIO.PortName.C)) ^ (1 << 7)));
								break;
							case GMasterJ7.PinName.LED1:
								CEPorts.WritePort("LED1:", sender.Value == 0 ? (byte)1 : (byte)0);
								break;
							case GMasterJ7.PinName.LED2:
								CEPorts.WritePort("LED2:", sender.Value == 0 ? (byte)1 : (byte)0);
								break;
							default:
								break;
						}
						#endregion
					}
					catch(Exception ex)
					{
						Console.WriteLine("Exception: " + ex.Message);
					}
				}
			}
			else
			{
				sender.DisplayValue = !sender.DisplayValue;
			}
		}

		protected override void Dispose( bool disposing )
		{
			// shut down timer before closing
			appTimer.Enabled = false;
			base.Dispose( disposing );
		}
		
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.sioPanel = new System.Windows.Forms.Panel();
			this.devicepartLabel = new System.Windows.Forms.Label();
			this.deviceversionLabel = new System.Windows.Forms.Label();
			this.firmwareLabel = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			// 
			// sioPanel
			// 
			this.sioPanel.Controls.Add(this.devicepartLabel);
			this.sioPanel.Controls.Add(this.deviceversionLabel);
			this.sioPanel.Controls.Add(this.firmwareLabel);
			this.sioPanel.Controls.Add(this.label3);
			this.sioPanel.Controls.Add(this.label5);
			this.sioPanel.Controls.Add(this.label4);
			this.sioPanel.Controls.Add(this.label1);
			this.sioPanel.Location = new System.Drawing.Point(392, 8);
			this.sioPanel.Size = new System.Drawing.Size(232, 336);
			// 
			// devicepartLabel
			// 
			this.devicepartLabel.Location = new System.Drawing.Point(144, 60);
			this.devicepartLabel.Size = new System.Drawing.Size(76, 16);
			this.devicepartLabel.Text = "(unknown)";
			// 
			// deviceversionLabel
			// 
			this.deviceversionLabel.Location = new System.Drawing.Point(144, 44);
			this.deviceversionLabel.Size = new System.Drawing.Size(76, 16);
			this.deviceversionLabel.Text = "(unknown)";
			// 
			// firmwareLabel
			// 
			this.firmwareLabel.Location = new System.Drawing.Point(144, 28);
			this.firmwareLabel.Size = new System.Drawing.Size(76, 16);
			this.firmwareLabel.Text = "(unknown)";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(12, 28);
			this.label3.Size = new System.Drawing.Size(132, 16);
			this.label3.Text = "Firmware Version:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(12, 60);
			this.label5.Size = new System.Drawing.Size(132, 16);
			this.label5.Text = "Device Part Type:";
			this.label5.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(12, 44);
			this.label4.Size = new System.Drawing.Size(132, 16);
			this.label4.Text = "Device Version:";
			this.label4.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label1.Location = new System.Drawing.Point(12, 8);
			this.label1.Size = new System.Drawing.Size(208, 20);
			this.label1.Text = "ADSmartIO Info";
			this.label1.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// Form1
			// 
			this.ClientSize = new System.Drawing.Size(626, 407);
			this.Controls.Add(this.sioPanel);
			this.Text = "ADSLibrary Test Application";
			this.Load += new System.EventHandler(this.Form1_Load);

		}
		#endregion

		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			deviceversionLabel.Text = Convert.ToString((char)ADSmartIO.DeviceVersion());
			devicepartLabel.Text = Convert.ToString(ADSmartIO.DevicePartType());
			firmwareLabel.Text = Convert.ToString(ADSmartIO.FirmwareVersion());
		}
		
	}
}
